@@ -1,14 +1,21 @@
 import traceback
+import warnings
 
 import click
-from docutils import nodes, statemachine
+from docutils import nodes
 from docutils.parsers import rst
 from docutils.parsers.rst import directives
+from docutils import statemachine
 from sphinx.util import logging
+from sphinx.util import nodes as sphinx_nodes
 
 LOG = logging.getLogger(__name__)
 CLICK_VERSION = tuple(int(x) for x in click.__version__.split('.')[0:2])
 
+NESTED_FULL = 'full'
+NESTED_SHORT = 'short'
+NESTED_NONE = 'none'
+
 
 def _indent(text, level=1):
     prefix = ' ' * (4 * level)
@@ -38,39 +45,58 @@ def _get_help_record(opt):
 
     [1] http://www.sphinx-doc.org/en/stable/domains.html#directive-option
     """
+
     def _write_opts(opts):
         rv, _ = click.formatting.join_options(opts)
         if not opt.is_flag and not opt.count:
-            rv += ' <{}>'.format(opt.name)
+            name = opt.name
+            if opt.metavar:
+                name = opt.metavar.lstrip('<[{($').rstrip('>]})$')
+            rv += ' <{}>'.format(name)
         return rv
 
     rv = [_write_opts(opt.opts)]
     if opt.secondary_opts:
         rv.append(_write_opts(opt.secondary_opts))
 
-    help = opt.help or ''
-    extra = []
+    out = []
+    if opt.help:
+        if opt.required:
+            out.append('**Required** %s' % opt.help)
+        else:
+            out.append(opt.help)
+    else:
+        if opt.required:
+            out.append('**Required**')
+
+    extras = []
+
     if opt.default is not None and opt.show_default:
         if isinstance(opt.show_default, str):
             # Starting from Click 7.0 this can be a string as well. This is
             # mostly useful when the default is not a constant and
             # documentation thus needs a manually written string.
-            extra.append('default: %s' % opt.show_default)
+            extras.append(':default: %s' % opt.show_default)
         else:
-            extra.append('default: %s' %
-                         (', '.join('%s' % d for d in opt.default) if isinstance(
-                         opt.default, (list, tuple)) else opt.default, ))
-    if opt.required:
-        extra.append('required')
-    if extra:
-        help = '%s[%s]' % (help and help + '  ' or '', '; '.join(extra))
-    if isinstance(opt.type, click.Choice):
-        help = "%s\n\n:options: %s" % (
-            help and help + "  " or "",
-            "|".join(opt.type.choices),
-        )
+            extras.append(
+                ':default: %s'
+                % (
+                    ', '.join('%s' % d for d in opt.default)
+                    if isinstance(opt.default, (list, tuple))
+                    else opt.default,
+                )
+            )
 
-    return ', '.join(rv), help
+    if isinstance(opt.type, click.Choice):
+        extras.append(':options: %s' % ' | '.join(opt.type.choices))
+
+    if extras:
+        if out:
+            out.append('')
+
+        out.extend(extras)
+
+    return ', '.join(rv), '\n'.join(out)
 
 
 def _format_description(ctx):
@@ -84,9 +110,9 @@ def _format_description(ctx):
         return
 
     bar_enabled = False
-    for line in statemachine.string2lines(help_string,
-                                          tab_width=4,
-                                          convert_whitespace=True):
+    for line in statemachine.string2lines(
+        help_string, tab_width=4, convert_whitespace=True
+    ):
         if line == '\b':
             bar_enabled = True
             continue
@@ -113,9 +139,9 @@ def _format_option(opt):
     yield '.. option:: {}'.format(opt[0])
     if opt[1]:
         yield ''
-        for line in statemachine.string2lines(opt[1],
-                                              tab_width=4,
-                                              convert_whitespace=True):
+        for line in statemachine.string2lines(
+            opt[1], tab_width=4, convert_whitespace=True
+        ):
             yield _indent(line)
 
 
@@ -123,8 +149,9 @@ def _format_options(ctx):
     """Format all `click.Option` for a `click.Command`."""
     # the hidden attribute is part of click 7.x only hence use of getattr
     params = [
-        x for x in ctx.command.params
-        if isinstance(x, click.Option) and not getattr(x, 'hidden', False)
+        param
+        for param in ctx.command.params
+        if isinstance(param, click.Option) and not getattr(param, 'hidden', False)
     ]
 
     for param in params:
@@ -137,9 +164,11 @@ def _format_argument(arg):
     """Format the output of a `click.Argument`."""
     yield '.. option:: {}'.format(arg.human_readable_name)
     yield ''
-    yield _indent('{} argument{}'.format(
-        'Required' if arg.required else 'Optional',
-        '(s)' if arg.nargs != 1 else ''))
+    yield _indent(
+        '{} argument{}'.format(
+            'Required' if arg.required else 'Optional', '(s)' if arg.nargs != 1 else ''
+        )
+    )
 
 
 def _format_arguments(ctx):
@@ -195,12 +224,29 @@ def _format_subcommand(command):
 
     if short_help:
         yield ''
-        for line in statemachine.string2lines(short_help,
-                                              tab_width=4,
-                                              convert_whitespace=True):
+        for line in statemachine.string2lines(
+            short_help, tab_width=4, convert_whitespace=True
+        ):
             yield _indent(line)
 
 
+def _format_epilog(ctx):
+    """Format the epilog for a given `click.Command`.
+
+    We parse this as reStructuredText, allowing users to embed rich
+    information in their help messages if they so choose.
+    """
+    epilog_string = ctx.command.epilog
+    if not epilog_string:
+        return
+
+    for line in statemachine.string2lines(
+        epilog_string, tab_width=4, convert_whitespace=True
+    ):
+        yield line
+    yield ''
+
+
 def _get_lazyload_commands(multicommand):
     commands = {}
     for command in multicommand.list_commands(multicommand):
@@ -222,7 +268,7 @@ def _filter_commands(ctx, commands=None):
     return [lookup[name] for name in names if name in lookup]
 
 
-def _format_command(ctx, show_nested, commands=None):
+def _format_command(ctx, nested, commands=None):
     """Format the output of `click.Command`."""
     if CLICK_VERSION >= (7, 0) and ctx.command.hidden:
         return
@@ -271,8 +317,13 @@ def _format_command(ctx, show_nested, commands=None):
     for line in lines:
         yield line
 
+    # description
+
+    for line in _format_epilog(ctx):
+        yield line
+
     # if we're nesting commands, we need to do this slightly differently
-    if show_nested:
+    if nested in (NESTED_FULL, NESTED_NONE):
         return
 
     commands = _filter_commands(ctx, commands)
@@ -291,14 +342,29 @@ def _format_command(ctx, show_nested, commands=None):
         yield ''
 
 
+def nested(argument):
+    values = (NESTED_FULL, NESTED_SHORT, NESTED_NONE)
+    if not argument:
+        return None
+
+    if argument not in values:
+        raise ValueError(
+            "%s is not a valid value for ':nested:'; allowed values: %s"
+            % directives.format_values(values)
+        )
+
+    return argument
+
+
 class ClickDirective(rst.Directive):
 
     has_content = False
     required_arguments = 1
     option_spec = {
         'prog': directives.unchanged_required,
-        'show-nested': directives.flag,
+        'nested': nested,
         'commands': directives.unchanged,
+        'show-nested': directives.flag,
     }
 
     def _load_module(self, module_path):
@@ -311,39 +377,38 @@ class ClickDirective(rst.Directive):
             module_name, attr_name = module_path.split(':', 1)
         except ValueError:  # noqa
             raise self.error(
-                '"{}" is not of format "module:parser"'.format(module_path))
+                '"{}" is not of format "module:parser"'.format(module_path)
+            )
 
         try:
             mod = __import__(module_name, globals(), locals(), [attr_name])
         except (Exception, SystemExit) as exc:  # noqa
-            err_msg = 'Failed to import "{}" from "{}". '.format(
-                attr_name, module_name)
+            err_msg = 'Failed to import "{}" from "{}". '.format(attr_name, module_name)
             if isinstance(exc, SystemExit):
                 err_msg += 'The module appeared to call sys.exit()'
             else:
                 err_msg += 'The following exception was raised:\n{}'.format(
-                    traceback.format_exc())
+                    traceback.format_exc()
+                )
 
             raise self.error(err_msg)
 
         if not hasattr(mod, attr_name):
-            raise self.error('Module "{}" has no attribute "{}"'.format(
-                module_name, attr_name))
+            raise self.error(
+                'Module "{}" has no attribute "{}"'.format(module_name, attr_name)
+            )
 
         parser = getattr(mod, attr_name)
 
         if not isinstance(parser, click.BaseCommand):
-            raise self.error('"{}" of type "{}" is not derived from '
-                             '"click.BaseCommand"'.format(
-                                 type(parser), module_path))
+            raise self.error(
+                '"{}" of type "{}" is not derived from '
+                '"click.BaseCommand"'.format(type(parser), module_path)
+            )
         return parser
 
-    def _generate_nodes(self,
-                        name,
-                        command,
-                        parent=None,
-                        show_nested=False,
-                        commands=None):
+    def _generate_nodes(self, name, command, parent, nested, commands=None,
+                        semantic_group=False):
         """Generate the relevant Sphinx nodes.
 
         Format a `click.Group` or `click.Command`.
@@ -351,9 +416,11 @@ class ClickDirective(rst.Directive):
         :param name: Name of command, as used on the command line
         :param command: Instance of `click.Group` or `click.Command`
         :param parent: Instance of `click.Context`, or None
-        :param show_nested: Whether subcommands should be included in output
+        :param nested: The granularity of subcommand details.
         :param commands: Display only listed commands or skip the section if
             empty
+        :param semantic_group: Display command as title and description for
+            CommandCollection.
         :returns: A list of nested docutil nodes
         """
         ctx = click.Context(command, info_name=name, parent=parent)
@@ -367,28 +434,40 @@ class ClickDirective(rst.Directive):
             '',
             nodes.title(text=name),
             ids=[nodes.make_id(ctx.command_path)],
-            names=[nodes.fully_normalize_name(ctx.command_path)])
+            names=[nodes.fully_normalize_name(ctx.command_path)],
+        )
 
         # Summary
-
         source_name = ctx.command_path
         result = statemachine.ViewList()
 
-        lines = _format_command(ctx, show_nested, commands)
+        if semantic_group:
+            lines = _format_description(ctx)
+        else:
+            lines = _format_command(ctx, nested, commands)
+
         for line in lines:
             LOG.debug(line)
             result.append(line, source_name)
 
-        self.state.nested_parse(result, 0, section)
+        sphinx_nodes.nested_parse_with_titles(self.state, result, section)
 
         # Subcommands
 
-        if show_nested:
-            commands = _filter_commands(ctx, commands)
-            for command in commands:
-                section.extend(
-                    self._generate_nodes(command.name, command, ctx,
-                                         show_nested))
+        if nested == NESTED_FULL:
+            if isinstance(command, click.CommandCollection):
+                for source in command.sources:
+                    section.extend(self._generate_nodes(
+                        source.name, source, parent=ctx, nested=nested,
+                        semantic_group=True
+                    ))
+            else:
+                commands = _filter_commands(ctx, commands)
+                for command in commands:
+                    parent = ctx if not semantic_group else ctx.parent
+                    section.extend(self._generate_nodes(
+                        command.name, command, parent=parent, nested=nested
+                    ))
 
         return [section]
 
@@ -402,10 +481,23 @@ class ClickDirective(rst.Directive):
 
         prog_name = self.options.get('prog')
         show_nested = 'show-nested' in self.options
+        nested = self.options.get('nested')
+
+        if show_nested:
+            if nested:
+                raise self.error(
+                    "':nested:' and ':show-nested:' are mutually exclusive"
+                )
+            else:
+                warnings.warn(
+                    "':show-nested:' is deprecated; use ':nested: full'",
+                    DeprecationWarning,
+                )
+                nested = NESTED_FULL if show_nested else NESTED_SHORT
+
         commands = self.options.get('commands')
 
-        return self._generate_nodes(prog_name, command, None, show_nested,
-                                    commands)
+        return self._generate_nodes(prog_name, command, None, nested, commands)
 
 
 def setup(app):
